package HSPC::MT::Plugin::PM::BT_DTAUS;

use strict;
use HSPC::PluginToolkit::General qw(throw_exception string);
use HSPC::MT::Plugin::PM::BT_DTAUS::Valid;


sub provided_payment_method_types {
	my $class = shift;
	return [
		{type => 'BT_DTAUS', title_id => 'dtaus_pm_type', image_id => ''},
	];
}

sub title_id {
	return 'dtaus_pm_ba';
}

sub get_public_data {
	my $class = shift;
	my %h = (
		data             => undef,
		@_
	);

	my %public_data = (
		holder_name    => $h{data}->{holder_name},
	);
	return \%public_data;
}

sub get_secure_data {
	my $class = shift;
	my %h = (
		data             => undef,
		keep_secure_code => undef,
		@_
	);

	my %secure_data = (
		account_number => $h{data}->{account_number},
		bank_code      => $h{data}->{bank_code},
	);
	return \%secure_data;
}

sub get_public_number {
	my $class = shift;
	my %h = (
		secure_data => undef,
		public_data => undef,
		@_
	);

	return substr($h{secure_data}->{account_number},-4);
}

sub get_secure_number {
	my $class = shift;
	my %h = (
		secure_data => undef,
		public_data => undef,
		@_
	);

	return $h{secure_data}->{bank_code}.$h{secure_data}->{account_number};
}

sub validate {
	my $class = shift;
	my %h = (
		expire_date => undef,
		secure_data => undef,
		public_data => undef,
		type        => undef,
		@_
	);

	my @errors;

	my $holder_name	= $h{public_data}->{holder_name};
	if( length($holder_name) > 27 ) {
		## do nothing here, but while batch generation such holder name will be cutted to 27 symbols
	}

	if ($h{secure_data}) {
		my $acc_number	= $h{secure_data}->{account_number};
		unless( length($acc_number) ) {
			push( @errors, string('dtaus_pm_account_number_must_be_defined') );
		}
		if( $acc_number =~ /\D/ ) {
			push( @errors, string('dtaus_pm_account_number_non_digit', acc=>$acc_number) );
		}
		if( length($acc_number) > 10 ) {
			push( @errors, string('dtaus_pm_account_number_length10') );
		}
	
		my $bank_code	= $h{secure_data}->{bank_code};
		unless( length($bank_code) ) {
			push( @errors, string('dtaus_pm_bank_code_must_be_defined') );
		}
		if( length($bank_code) > 8 ) {
			push( @errors, string('dtaus_pm_bank_code_length8') );
		}
	
		unless( scalar(@errors) ) {
			my $validator	= HSPC::MT::Plugin::PM::BT_DTAUS::Valid->new();
			push( @errors, string('dtaus_pm_bank_code_wrong') )
				unless $validator->IsAccountValid($acc_number,$bank_code);
		}
	}

	throw_exception(
		type    => 'parameter',
		message => join( "<br>\n", @errors ),
	) if scalar(@errors);

	return undef;
}

sub get_paymethod_type {
	return 'BT_DTAUS';
}

1;
