package HSPC::MT::Plugin::PM::BT_DTAUS::Valid;

use strict;
use HSPC::PluginToolkit::General qw(string);
use HSPC::MT::Plugin::PM::BT_DTAUS::BLZS;
use POSIX;


sub new {
	my $class = shift;
	my $self  = {};
	bless ($self,$class);
	return $self;
}

sub CrossSum {
	my $self = shift;
	my $dig  = shift;
	$dig = substr($dig,0,1) + substr($dig,1,1) if ($dig > 9);
	return $dig;
}

sub ExpandAccount {
	my $self = shift;
	my $acc  = shift;
	(length($acc) < 10) ?
		substr($acc,0,-9) = '0'x(10-length($acc)) :
		substr($acc,0,-10) = '';
	return $acc;
}

sub GetPrz {
	my $self = shift;
	my $blz  = shift;

	return	HSPC::MT::Plugin::PM::BT_DTAUS::BLZS->method_id( blz => $blz );
}

## checks account
sub IsAccountValid {
	my $self = shift;
	my $acc  = shift;
	my $blz  = shift;

	return undef	unless $acc;
	return undef	unless $blz;
	my $prz = $self->GetPrz($blz);
	return undef	unless $prz;
	my $ret = undef;
	if ($prz eq 52 || $prz eq 53) {
		$ret = eval '$self->Mark'.$prz.'('.$acc.','.$blz.');';
	} elsif ($prz eq 87) { ## Method is erroneous
		$ret = 0;
	} else {
		$ret = eval '$self->Mark'.$prz.'('.$acc.');';
	}
	return ($ret > 0) ? undef : 1;
}

##################################
##### Methods start ##############
##################################

sub Method00 {
	my $self  = shift;
	my $acc   = shift;
	my $sig   = shift;
	my $check = shift;
	my $mod   = shift || 10;

	my $method00 = 1;
	$acc = $self->ExpandAccount($acc);
	my $h = 0;
	$h += $self->CrossSum(substr($acc,$_,1) * substr($sig,$_,1))
		for (0..(length($sig)-1));
	$h %= $mod;
	my $checksum = $mod - $h;
	$checksum = 0 if ($checksum == $mod);
	$method00 = 0 if ($checksum eq substr($acc,$check-1,1));
	return $method00;
}

sub Method01 {
	my $self  = shift;
	my $acc   = shift;
	my $sig   = shift;

    my $h = 0;
	my $method01 = 1;
	$acc = $self->ExpandAccount($acc);
	$h += substr($acc,$_,1) * substr($sig,$_,1) for (0..(length($sig)-1));
	$h %= 10;
	my $checksum = 10 - $h;

	$checksum = 0 if ($checksum == 10);
	$method01 = 0 if ($checksum == substr($acc,-1));
	return $method01;
}

sub Method02 {
	my $self  = shift;
	my $acc   = shift;
	my $sig   = shift;
	my $modif = shift;

    my $h = 0;
	my $method02 = 1;
	$acc = $self->ExpandAccount($acc);
	if ($modif) {
		$h += substr($acc,$_,1) * hex(substr($sig,$_,1)) for (0..(length($sig)-1));
	} else {
		$h += substr($acc,$_,1) * substr($sig,$_,1) for (0..(length($sig)-1));
	}
    $h %= 11;
	$h = 11 if ($h == 0);
    if ($h != 1) {
		my $checksum = 11 - $h;
		if ($checksum == substr($acc, -1)) {
			$method02 = 0;
		}
    }
	return $method02;
}

sub Method06 {
	my $self  = shift;
	my $acc   = shift;
	my $sig   = shift;
	my $modif = shift;
	my $check = shift;
	my $mod   = shift;

	my $h = 0;
	my $method06 = 1;
	$acc = $self->ExpandAccount($acc);
	if ($modif) {
		$h += substr($acc,$_,1) * hex(substr($sig,$_,1)) for (0..(length($sig)-1));
	} else {
		$h += substr($acc,$_,1) * substr($sig,$_,1) for (0..(length($sig)-1));
	}
	$h %= $mod;
	my $checksum = $mod - $h;
	$checksum = 0 if ($h < 2 && $mod == 11);
	$method06 = 0 if ($checksum == substr($acc,$check-1,1));
	return $method06;
}

sub Method16 {
	my $self  = shift;
	my $acc   = shift;
	my $sig   = shift;
	my $check = shift;

	my $h = 0;
	my $method16 = 1;
    $acc = $self->ExpandAccount($acc);
	$h += substr($acc,$_,1) * substr($sig,$_,1) for (0..(length($sig)-1));
    $h %= 11;
    my $checksum = 11 - $h;
	$checksum = 0 if ($h == 0);
    if ($checksum == substr($acc,$check-1,1)) {
		$method16 = 0;
	}
	if ($h == 1) {
		if ($checksum == substr($acc,$check - 2,1)) {
			$method16 = 0;
		}
	}
	return $method16;
}

sub Method90 {
	my $self  = shift;
	my $acc   = shift;
	my $sig   = shift;
	my $check = shift;
	my $mod   = shift;

    my $h = 0;
    my $method90 = 1;
    $acc = $self->ExpandAccount($acc);
	$h += substr($acc,$_,1) * substr($sig,$_,1) for (0..(length($sig)-1));
    $h %= $mod;
    my $checksum = $mod - $h;
	$checksum = 0 if ($h == 0);
    if ($checksum == substr($acc,$check -1, 1)) {
		$method90 = 0;
    }
    return $method90;
}

##################################
##### Methods end ################
##################################

##################################
##### Marks start ################
##################################


sub Mark00 {
	my $self = shift;
	my $acc  = shift;
	my $mark00 = $self->Method00($acc, '212121212', 10);
	return $mark00;
}

sub Mark01 {
	my $self = shift;
	my $acc  = shift;
	my $mark01 = $self->Method01($acc, '173173173');
	return $mark01;
}

sub Mark02 {
	my $self = shift;
	my $acc  = shift;
    my $mark02 = $self->Method02($acc, '298765432', 0);
    return $mark02;
}

sub Mark03 {
	my $self = shift;
	my $acc  = shift;
    my $mark03 = $self->Method01($acc, '212121212');
    return $mark03;
}

sub Mark04 {
	my $self = shift;
	my $acc  = shift;
    my $mark04 = $self->Method02($acc, '432765432', 0);
    return $mark04;
}

sub Mark05 {
	my $self = shift;
	my $acc  = shift;
	my $mark05 = $self->Method01($acc, '137137137');
	return $mark05;
}

sub Mark06 {
	my $self = shift;
	my $acc  = shift;
    my $mark06 = $self->Method06($acc, '432765432', 0, 10, 11);
    return $mark06;
}

sub Mark07 {
	my $self = shift;
	my $acc  = shift;
    my $mark07 = $self->Method02($acc, 'A98765432', 1);
    return $mark07;
}

sub Mark08 {
	my $self = shift;
	my $acc  = shift;
	my $mark08 = 1;
	if ($acc > 60000) {
		my $mark08 = $self->Method00($acc, '212121212', 10);
	}
    return $mark08;
}

sub Mark09 {
	my $self = shift;
	my $acc  = shift;
    my $mark09 = 0;
    return $mark09;
}

sub Mark10 {
	my $self = shift;
	my $acc  = shift;
    my $mark10 = $self->Method06($acc, 'A98765432', 1, 10, 11);
    return $mark10;
}

sub Mark11 {
	my $self = shift;
	my $acc  = shift;
    my $sig = 'A98765432';
    my $h = 0;
    my $mark11 = 1;
    $acc = $self->ExpandAccount($acc);
	$h += substr($acc,$_,1) * hex(substr($sig,$_,1)) for (0..(length($sig)-1));
    $h %= 11;
    my $checksum = 11 - $h;
	$checksum = 0 if ($h == 0);
	$checksum = 9 if ($h == 1);
	$mark11 = 0 if ($checksum == substr($acc,-1));
    return $mark11;
}

sub Mark12 {
	my $self = shift;
	my $acc  = shift;
	my $mark12 = $self->Method01($acc, '731731731');
    return $mark12;
}

sub Mark13 {
	my $self = shift;
	my $acc  = shift;
    my $h = $self->Method00($acc, '0121212', 8);
    if ($h == 1) {
		if (substr($acc,-2) ne '00') {
			$h = $self->Method00(substr($self->ExpandAccount($acc), 2) . '00', '0121212', 8);
		}
    }
    my $mark13 = $h;
    return $mark13;
}

sub Mark14 {
	my $self = shift;
	my $acc  = shift;
    my $mark14 = $self->Method02($acc, '000765432', 0);
    return $mark14;
}

sub Mark15 {
	my $self = shift;
	my $acc  = shift;
    my $mark15 = $self->Method06($acc, '000005432', 0, 10, 11);
    return $mark15;
}

sub Mark16 {
	my $self = shift;
	my $acc  = shift;
    my $mark16 = $self->Method16($acc, '432765432', 10);
    return $mark16;
}

sub Mark17 {
	my $self = shift;
	my $acc  = shift;
    my $sig = '0121212';
    my $h = 0;
    my $h2 = 1;
    $acc = $self->ExpandAccount($acc);
	$h += ($self->CrossSum(substr($acc,$_,1) * substr($sig,$_,1))) for (0..(length($sig)-1));
    $h = $h - 1;
    my $checksum = $h % 11;
    $checksum = 10 - $checksum;
	$checksum = 0 if ($checksum == 10);
	$h2 = 0 if ($checksum == substr($acc,7,1));
    my $mark17 = $h2;
    return $mark17;
}

sub Mark18 {
	my $self = shift;
	my $acc  = shift;
    my $mark18 = $self->Method01($acc, '317931793');
    return $mark18;
}

sub Mark19 {
	my $self = shift;
	my $acc  = shift;
    my $mark19 = $self->Method06($acc, '198765432', 0, 10, 11);
    return $mark19;
}

sub Mark20 {
	my $self = shift;
	my $acc  = shift;
    my $mark20 = $self->Method06($acc, '398765432', 0, 10, 11);
    return $mark20;
}

sub Mark21 {
	my $self = shift;
	my $acc  = shift;
    my $sig = '212121212';
    my $h = 0;
    my $mark21 = 1;
    $acc = $self->ExpandAccount($acc);
	$h += ($self->CrossSum(substr($acc,$_,1) * substr($sig,$_,1))) for (0..(length($sig)-1));
    $h = $self->CrossSum($h);
	my $checksum = 10 - $h;
	$checksum = 0 if ($checksum == 10);
	$mark21 = 0 if ($checksum == substr($acc,-1));
	return $mark21;
}

sub Mark22 {
	my $self = shift;
	my $acc  = shift;
    my $sig = '313131313';
    my $h = 0;
    my $mark22 = 1;
    $acc = $self->ExpandAccount($acc);
	$h += (substr($acc,$_,1) * substr($sig,$_,1)) % 10 for (0..(length($sig)-1));
	my $checksum = ceil($h/10)*10 - $h;
	if ($checksum == substr($acc,-1)) {
		$mark22 = 0;
    }
    return $mark22;
}

sub Mark23 {
	my $self = shift;
	my $acc  = shift;
    my $mark23 = $self->Method16($acc, '765432', 7);
    return $mark23;
}

sub Mark24 {
	my $self = shift;
	my $acc  = shift;
    my $sig = '123123123';
    my $h = 0;
    my $mark24 = 1;
	$acc =~ s/^0*//g;
	$h += (substr($acc,$_,1) * substr($sig,$_,1) + substr($sig,$_,1)) % 11 for (0..(length($acc)-2));
	my $checksum = $h % 10;
	$mark24 = 0 if ($checksum == substr($acc,-1));
    return $mark24;
}

sub Mark25 {
	my $self = shift;
	my $acc  = shift;
    my $sig = '098765432';
    my $Falsch = 0;
    my $h = 0;
    $acc = $self->ExpandAccount($acc);
	$h += (substr($acc,$_,1) * substr($sig,$_,1)) for (0..(length($sig)-1));
    $h %= 11;
    my $checksum = 11 - $h;
	$checksum = 0 if ($checksum == 11);
	my $mark25;
    if ($checksum == 10) {
		$checksum = 0;
		if ((substr($acc,1,1) ne '8') and (substr($acc,1,1) ne '9')) {
			$mark25 = 1;
			$Falsch = 1;
		}
    }
    if ($Falsch == 0) {
		if ($checksum == substr($acc,-1)) {
			$mark25 = 0;
		} else {
			$mark25 = 1;
		}
    }
    return $mark25;
}

sub Mark26 {
	my $self = shift;
	my $acc  = shift;
    $acc = $self->ExpandAccount($acc);
    if (substr($acc,-2) == '00') {
		$acc = substr($acc,2) . '00';
    }
    my $mark26 = $self->Method06($acc,'2345672',0,8,11);
    return $mark26;
}

sub Mark27 {
	my $self = shift;
	my $acc  = shift;
	my $mark27;
    if (int($acc) < 1000000000) {
		$mark27 = $self->Method00($acc, '212121212', 10);
    } else {
		$mark27 = $self->Mark29($acc);
    }
    return $mark27;
}


sub Mark28 {
	my $self = shift;
	my $acc  = shift;
    my $mark28 = $self->Method06($acc, '8765432', 0, 8, 11);
    return $mark28;
}

sub Mark29 {
	my $self = shift;
	my $acc  = shift;
    my $Transform = '143214321';
    $acc = $self->ExpandAccount($acc);
    my $mark29 = 1;
    my $h = 0;
	my $replace =
	{
		1 => [0,1,5,9,3,7,4,8,2,6],
		2 => [0,1,7,6,9,8,3,2,5,4],
		3 => [0,1,8,4,6,2,9,5,7,3],
		4 => [0,1,2,3,4,5,6,7,8,9]
	};
	for (0..(length($Transform)-1)) {
		$h += $replace->{substr($Transform,$_,1)}->[substr($acc,$_,1)];
	}
    $h %= 10;
    my $checksum = 10 - $h;
	$mark29 = 0 if ($checksum = substr($acc,-1));
    return $mark29;
}

sub Mark30 {
	my $self = shift;
	my $acc  = shift;
    my $sig = '200001212';
    my $h = 0;
    my $mark30 = 1;
    $acc = $self->ExpandAccount($acc);
	$h += (substr($acc,$_,1) * substr($sig,$_,1)) for (0..(length($sig)-1));
    $h %= 10;
    my $checksum = 10 - $h;
	$checksum = 0 if ($checksum == 10);
	$mark30 = 0 if ($checksum == substr($acc,-1));
    return $mark30;
}

sub Mark31 {
	my $self = shift;
	my $acc  = shift;
    my $sig = '123456789';
    my $h = 0;
    my $mark31 = 1;
    $acc = $self->ExpandAccount($acc);
	$h += (substr($acc,$_,1) * substr($sig,$_,1)) for (0..(length($sig)-1));;
    my $checksum = $h % 11;
	$mark31 = 0 if ($checksum == substr($acc,-1));
    return $mark31;
}


sub Mark32 {
	my $self = shift;
	my $acc  = shift;
    my $mark32 = $self->Method06($acc, '000765432', 0, 10, 11);
    return $mark32;
}

sub Mark33 {
	my $self = shift;
	my $acc  = shift;
    my $mark33 = $self->Method06($acc, '000065432', 0, 10, 11);
    return $mark33;
}

sub Mark34 {
	my $self = shift;
	my $acc  = shift;
    my $mark34 = $self->Method06($acc, '79A5842', 1, 8, 11);
    return $mark34;
}

sub Mark35 {
	my $self = shift;
	my $acc  = shift;
    my $mark35 = 3;
    return $mark35;
}

sub Mark36 {
	my $self = shift;
	my $acc  = shift;
    my $mark36 = $self->Method06($acc, '000005842', 0, 10, 11);
    return $mark36;
}

sub Mark37 {
	my $self = shift;
	my $acc  = shift;
    my $mark37 = $self->Method06($acc, '0000A5842', 1, 10, 11);
    return $mark37;
}

sub Mark38 {
	my $self = shift;
	my $acc  = shift;
    my $mark38 = $self->Method06($acc, '0009A5842', 1, 10, 11);
    return $mark38;
}

sub Mark39 {
	my $self = shift;
	my $acc  = shift;
    my $mark39 = $self->Method06($acc, '0079A5842', 1, 10, 11);
    return $mark39;
}

sub Mark40 {
	my $self = shift;
	my $acc  = shift;
    my $mark40 = $self->Method06($acc, '6379A5842', 1, 10, 11);
    return $mark40;
}

sub Mark41 {
	my $self = shift;
	my $acc  = shift;
    $acc = $self->ExpandAccount($acc);
    if (substr($acc,3,1) == '9') {
		$acc = '000'. substr($acc,3);
    }
    my $mark41 = $self->Method00($acc,'212121212',10);
    return $mark41;
}

sub Mark42 {
	my $self = shift;
	my $acc  = shift;
    my $mark42 = $self->Method06($acc, '098765432', 0, 10, 11);
    return $mark42;
}

sub Mark43 {
	my $self = shift;
	my $acc  = shift;
    my $sig = '987654321';
    my $h = 0;
    my $mark43 = 1;
    $acc = $self->ExpandAccount($acc);
	$h += (substr($acc,$_,1) * substr($sig,$_,1)) for (0..(length($sig)-1));
    $h %= 10;
    my $checksum = 10 - $h;
	$checksum = 0 if ($checksum == 10);
	$mark43 = 0 if ($checksum == substr($acc,-1));
    return $mark43;
}

sub Mark44 {
	my $self = shift;
	my $acc  = shift;
    my $mark44 = $self->Method06($acc, '0000A5842', 1, 10, 11);
    return $mark44;
}

sub Mark45 {
	my $self = shift;
	my $acc  = shift;
	my $mark45;
    if (substr($acc,0,1)=='0' or substr($acc,4,1)=='1') {
		$mark45 = 2;
    } else {
		$mark45 = $self->Method00($acc, '212121212', 10);
    }
    return $mark45;
}

sub Mark46 {
	my $self = shift;
	my $acc  = shift;
    my $mark46 = $self->Method06($acc, '0065432', 0, 8, 11);
    return $mark46;
}

sub Mark47 {
	my $self = shift;
	my $acc  = shift;
    my $mark47 = $self->Method06($acc, '00065432', 0, 9, 11);
    return $mark47;
}

sub Mark48 {
	my $self = shift;
	my $acc  = shift;
    my $mark48 = $self->Method06($acc, '00765432', 0, 9, 11);
    return $mark48;
}

sub Mark49 {
	my $self = shift;
	my $acc  = shift;
    my $mark49 = $self->Mark00($acc);
	return $mark49 if ($mark49 == 0);
    $mark49 = $self->Mark01($acc);
    return $mark49;
}

sub Mark50 {
	my $self = shift;
	my $acc  = shift;
    my $h = $self->Method06($acc, '765432', 0, 7, 11);
    if ($h == 1) {
		if (length($acc) < 7) {
			$h = $self->Method06($acc . '000', '765432', 0, 7, 11);
		}
    }
    my $mark50 = $h;
    return $mark50;
}

sub Mark51 {
	my $self = shift;
	my $acc  = shift;
	$acc = $self->ExpandAccount($acc);
	my $h;
	if (substr($acc,2,1) ne '9') {
        $h = $self->Method06($acc, '000765432', 0, 10, 11);
		if ($h != 0) {
			$h = $self->Method06($acc, '000065432', 0, 10, 11);
			if ($h != 0) {
				my $val = substr($acc,-1);
				if ($val eq 7 || $val eq 8 || $val eq 9) {
					$h = 4;
				} else {
					$h = $self->Method06($acc, '000065432', 0, 10, 7);
				}
			}
		}
	} else {
        $h = $self->Method06($acc, '008765432', 0, 10, 11);
        if ($h != 0 ) {
			$h = $self->Method06($acc, 'A98765432', 1, 10, 11);
        }
	}
	return $h;
}

sub Mark52 {
	my $self = shift;
	my $acc  = shift;
	my $blz  = shift;
    my $sig = '4216379A5842';
	my $correct;
    if ((length($acc) == 10) && (substr($acc,0,1) == '9')) {
        $correct = $self->Mark20($acc);
    } else {
		my $h = 0;
		my $rest = 0;
		my $altkonto = substr($blz,-4).substr($acc,0,2);
		$acc = substr($acc,2);
		$acc =~ s/^0*//g;
		$altkonto .= $acc;
		my $checksum = substr($altkonto,5,1);
		$altkonto = substr($altkonto,0,5).'0'.substr($altkonto,6);
		my $laenge = length($altkonto);
		$sig = substr($sig,(12 - $laenge));
		$h += substr($altkonto,$_,1) * hex(substr($sig,$_,1)) for (0..($laenge-1));
		$rest = $h % 11;
		my $gewicht = hex(substr($sig,5,1));

		my $PZ = -1;
		my $h2;
		{
			do {
				$PZ++;
				$h2 = $rest + ($PZ * $gewicht);
				last if ($PZ == 9);
			} while ($h2 % 11 != 10);
		}
		if ($h2 % 11 == 10) {
			if ($PZ == $checksum) {
				$correct = 0;
			} else {
				$correct = 1;
			}
		} else {
			$correct = 1;
		}
	}
    return $correct;
}

sub Mark53 {
	my $self = shift;
	my $acc  = shift;
	my $blz  = shift;
    my $sig = '4216379A5842';
	my $correct;
    if (length($acc) == 10) {
		$correct = $self->Mark20($acc) if (substr($acc,0,1) == '9');
    } else {
		my $h = 0;
		my $Rest = 0;
		my $AltKonto = substr($blz,-4,2) . substr($acc,1,1) . substr($blz,-1). substr($acc,0,1) . substr($acc,2,1);
		$acc = substr($acc,3);
		while (substr($acc,0,1) == '0') {
			$acc = substr($acc,1);
		}
		$AltKonto = $AltKonto . $acc;
		while (length($AltKonto)<12) {
			$AltKonto = "0" . $AltKonto;
		}
		my $checksum = substr($AltKonto,5,1);
		$AltKonto = substr($AltKonto,0,5) . '0' . substr($AltKonto,6);
		$h += substr($AltKonto,$_,1) * hex(substr($sig,$_,1)) for (0..(length($AltKonto))-1);
		$Rest = $h % 11;
		my $Gewicht = hex(substr($sig,5,1));
		my $PZ = -1;
		my $h2;
		do {
			$PZ++;
			$h2 = $Rest + ($PZ * $Gewicht);
		} while ($h2 % 11 != 10 or $PZ > 9);

		if ($h2 % 11 == 10) {
			if ($PZ == $checksum) {
				$correct = 0;
			} else {
				$correct = 1;
			}
		} else {
			$correct = 1;
		}
    }
    return $correct;
}

sub Mark54 {
	my $self = shift;
	my $acc  = shift;
    my $mark54 = 3;
    return $mark54;
}

sub Mark55 {
	my $self = shift;
	my $acc  = shift;
    my $mark55 = $self->Method06($acc, '878765432', 0, 10, 11);
    return $mark55;
}

sub Mark56 {
	my $self = shift;
	my $acc  = shift;
    my $sig = '432765432';
    my $mark56 = 1;
    my $h = 0;
    my $Correct = 0;
    $acc = $self->ExpandAccount($acc);
	$h += (substr($acc,$_,1) * substr($sig,$_,1)) for (0..(length($sig))-1);
    $h %= 11;
    $h = 11 - $h;
    my $checksum = $h;
	if (substr($acc,1,1) eq '9') {
		$checksum = 8 if ($h == 11);
		$checksum = 7 if ($h == 10);
	} else {
		$Correct = 1 if ($h == 11 || $h == 10);
	}
    if ($Correct == 0) {
		$mark56 = 0 if ($checksum == substr($acc,-1));
    }
    return $mark56;
}

sub Mark57 {
	my $self = shift;
	my $acc  = shift;
    my $Correct = 1;
    $acc = $self->ExpandAccount($acc);
    my $h = substr($acc,0,2);
	if ($h <= 50 || $h == 91 || ($h >= 96 && $h <= 99)) {
		return 0;
	}
	if ($acc =~ m/[87]{6}/) {
		return 0;
	}
    my $mark57 = $self->Method00($acc, '121212121', 10);
    return $mark57;
}

sub Mark58 {
	my $self = shift;
	my $acc  = shift;
    my $mark58 = $self->Method02($acc, '000065432', 0);
    return $mark58;
}

sub Mark59 {
	my $self = shift;
	my $acc  = shift;
    my $mark59 = 1;
	if (length($acc) > 8) {
        $mark59 = $self->Method00($acc, '212121212', 10);
	}
	return $mark59;
}

sub Mark60 {
	my $self = shift;
	my $acc  = shift;
    my $mark60 = $self->Method00($acc, '002121212', 10);
    return $mark60;
}

sub Mark61 {
	my $self = shift;
	my $acc  = shift;
    $acc = $self->ExpandAccount($acc);
	my $mark61;
    if (substr($acc,8,1) == '8') {
		$mark61 = $self->Method00($acc, '2121212012', 8);
    } else {
		$mark61 = $self->Method00($acc, '2121212', 8);
    }
    return $mark61;
}

sub Mark62 {
	my $self = shift;
	my $acc  = shift;
    my $mark62 = $self->Method00($acc,'0021212',8);
    return $mark62;
}

sub Mark63 {
	my $self = shift;
	my $acc  = shift;
    my $h = $self->Method00($acc,'0121212',8);
	$h = $self->Method00($acc,'000121212',10) if ($h == 1);
    my $mark63 = $h;
    return $mark63;
}

sub Mark64 {
	my $self = shift;
	my $acc  = shift;
    my $mark64 = $self->Method06($acc, '9A5842', 1, 7, 11);
    return $mark64;
}

sub Mark65 {
	my $self = shift;
	my $acc  = shift;
    $acc = $self->ExpandAccount($acc);
	my $mark65;
    if (substr($acc,8,1) == '9') {
		$mark65 = $self->Method00($acc, '2121212012', 8);
    } else {
		$mark65 = $self->Method00($acc, '2121212', 8);
    }
    return $mark65;
}

sub Mark66 {
	my $self = shift;
	my $acc  = shift;
    my $sig = '000065432';
    my $h = 0;
    my $mark66 = 1;
    $acc = $self->ExpandAccount($acc);
	$h += (substr($acc,$_,1) * substr($sig,$_,1)) for (0..(length($sig)-1));
    $h %= 11;
    my $checksum = 11 - $h;
	$checksum = $h if ($h == 0 || $h == 1);
	$mark66 = 0 if ($checksum == substr($acc,-1));
    return $mark66;
}

sub Mark67 {
	my $self = shift;
	my $acc  = shift;
    my $mark67 = $self->Method00($acc, '2121212', 8);
    return $mark67;
}

sub Mark68 {
	my $self = shift;
	my $acc  = shift;
    my $correct = 0;
    my $sig = '212121212';
    if (length($acc) == 9) {
		if (substr($acc,1,1) == '4') {
			$correct = 4;
		}
    }
    if (length($acc) == 10) {
		$sig = '000121212';
    }
    if ($correct == 0) {
		$correct = $self->Method00($acc,$sig,10);
		if ($correct == 1) {
			$correct = $self->Method00($acc,'210021212',10);
		}
    }
    my $mark68 = $correct;
    return $mark68;
}

sub Mark69 {
	my $self = shift;
	my $acc  = shift;
    $acc = $self->ExpandAccount($acc);
    my $correct = 0;
    if (substr($acc,0,2) == '93') {
		$correct = 2;
    }
    if ($correct == 0) {
		$correct = $self->Mark28($acc);
		if ($correct == 1) {
			$correct = $self->Mark29($acc);
		} elsif (substr($acc,0,2) == '97') {
			$correct = $self->Mark29($acc);
		}
    }
    my $mark69 = $correct;
    return $mark69;
}

sub Mark70 {
	my $self = shift;
	my $acc  = shift;
    $acc = $self->ExpandAccount($acc);
	my $mark70;
    if (substr($acc,3,1) == '5') {
		$mark70 = $self->Method06($acc, '000765432', 0, 10, 11);
    } elsif (substr($acc,3,2) == '69') {
		$mark70 = $self->Method06($acc, '000765432', 0, 10, 11);
    } else {
		$mark70 = $self->Method06($acc, '432765432', 0, 10, 11);
	}
    return $mark70;
}

sub Mark71 {
	my $self = shift;
	my $acc  = shift;
    my $sig='0654321';
    my $h = 0;
    my $mark71 = 1;
    $acc = $self->ExpandAccount($acc);
	$h += (substr($acc,$_,1) * substr($sig,$_,1)) for (0..(length($sig)-1));
    $h %= 11;
    my $checksum = 11 - $h;
	$checksum = $h if ($h == 0 || $h == 1);
    if ($checksum == substr($acc,-1)) {
		$mark71 = 0;
    }
    return $mark71;
}

sub Mark72 {
	my $self = shift;
	my $acc  = shift;
    my $mark72 = $self->Method00($acc, '000121212', 10);
    return $mark72;
}

sub Mark73 {
	my $self = shift;
	my $acc  = shift;
    $acc = $self->ExpandAccount($acc);
	my $mark73;
    if (substr($acc,2,1) != '9') {
		$mark73 = $self->Method00($acc, '000121212', 10);
		if ($mark73 != 0) {
			$mark73 = $self->Method00($acc, '000021212', 10);
			if ($mark73 != 0) {
				$mark73 = $self->Method00($acc, '000021212', 10, 7);
			}
		}
    } else {
		$mark73 = $self->Mark51($acc);
    }
    return $mark73;
}

sub Mark74 {
	my $self = shift;
	my $acc  = shift;
    my $h = 0;
    my $V2 = 0;
	$V2 = 1 if (length($acc) == 6);
    my $correct = $self->Method00($acc,'212121212',10);
    if ($correct == 1) {
		if ($V2 == 1) {
			my $sig = '212121212';
			$correct = 1;
			$acc = $self->ExpandAccount($acc);
			$h += $self->CrossSum(substr($acc,$_,1) * substr($sig,$_,1)) for (0..(length($sig)-1));
			$h %= 10;
			$h = 5 - $h;
			$h = 10 + $h if ($h < 0);
			my $checksum = $h;
			$correct = 0 if ($checksum == substr($acc,-1));
		}
    }
    return $correct;
}

sub Mark75 {
	my $self = shift;
	my $acc  = shift;
    my $h = 1;
	my $length = length($acc);
	if ($length == 6 || $length == 7) {
		$h = $self->Method00($acc,'000021212',10);
	} elsif ($length == 9) {
		if (substr($acc,0,1) eq '9') {
			$h = $self->Method00($acc,'0021212',8);
		} else {
			$h = $self->Method00($acc,'012121',7);
		}
	} elsif ($length == 10) {
		$h = $self->Method00($acc,'012121',7);
	}
    return $h;
}

sub Mark76 {
	my $self = shift;
	my $acc  = shift;
    my $h = 0;
    my $correct = 1;
    my $sig = '0765432';
    $acc = $self->ExpandAccount($acc);
	$h += substr($acc,$_,1) * substr($sig,$_,1) for (0..(length($sig)-1));
    $h = $h % 11;
    if ($h == 10) {
		$correct = 4;
    } else {
		if ($h eq substr($acc,-3,1)) {
			$correct = 0;
		} else {
			$h = 0;
			$sig = '000765432';
			$h += substr($acc,$_,1) * substr($sig,$_,1) for (0..(length($sig)-1));
			$h = $h % 11;
			if ($h == 10) {
				$correct = 4;
			} else {
				if ($h == substr($acc,-1)) {
					$correct = 0;
				} else {
					$correct = 1;
				}
			}
		}
    }
    return $correct;
}

sub Mark77 {
	my $self = shift;
	my $acc  = shift;
    my $h = 0;
    my $mark77 = 1;
    $acc = $self->ExpandAccount($acc);
    my $sig = '54321';
	$h += (substr($acc,$_,1) * substr($sig,$_,1)) for (0..4);
    $h %= 11;
    if ($h != 0) {
		$h = 0;
		$sig = '54345';
		$h += (substr($acc,$_,1) * substr($sig,$_,1)) for (0..4);
		$h %= 11;
		$mark77 = 0 if ($h == 0);
    }
    return $mark77;
}

sub Mark78 {
	my $self = shift;
	my $acc  = shift;
	my $mark78;
    if (length($acc) == 8) {
		$mark78 = 4;
    } else {
		$mark78 = $self->Method00($acc, '212121212', 10);
    }
    return $mark78;
}

sub Mark79 {
	my $self = shift;
	my $acc  = shift;
    $acc = $self->ExpandAccount($acc);
	my $h = substr($acc,0,1);
	my $mark79;
	if (map {$_ == $h} (0,7,8)) {
		$mark79 = 1;
	} elsif (map {$_ == $h} (1,2,9)) {
		$mark79 = $self->Method00($acc, '12121212', 9);
	} elsif (map {$_ == $h} (3,4,5,6)) {
		$mark79 = $self->Method00($acc, '212121212', 10);
	} else {
		$mark79 = 1;
	}
    return $mark79;
}

sub Mark80 {
	my $self = shift;
	my $acc  = shift;
    $acc = $self->ExpandAccount($acc);
	my $mark80;
    if (substr($acc,2,1) == '9') {
		$mark80 = $self->Mark51($acc);
    } else {
		$mark80 = $self->Method00($acc, '000021212', 10);
    }
    if ($mark80 != 0) {
		my $sig='000021212';
		my $h = 0;
		$mark80 = 1;
		$h += $self->CrossSum(substr($acc,$_,1) * substr($sig,$_,1)) for (0..(length($sig)-1));
		$h %= 7;
		my $checksum = 7 - $h;

		$checksum = 0 if ($checksum == 10);
		$mark80 = 0 if ($checksum == substr($acc,9,1));
    }
    return $mark80;
}

sub Mark81 {
	my $self = shift;
	my $acc  = shift;
  	$acc = $self->ExpandAccount($acc);
	my $mark81;
    if (substr($acc,2,1) == '9') {
		$mark81 = $self->Mark10($acc);
    } else {
		$mark81 = $self->Mark51($acc);
    }
    return $mark81;
}

sub Mark82 {
	my $self = shift;
	my $acc  = shift;
    $acc = $self->ExpandAccount($acc);
	my $mark82;
	if (substr($acc,2, 2) == '99') {
		$mark82 = $self->Mark10($acc);
	} else {
		$mark82 = $self->Mark33($acc);
	}
	return $mark82;
}

sub Mark83 {
	my $self = shift;
	my $acc  = shift;
    my $h = $self->Method06($acc, '000065432', 0, 10, 11);
    if ($h == 1) {
		$h = $self->Method06($acc, '000065432', 0, 10, 7);
    }
    my $mark83 = $h;
    return $mark83;
}

sub Mark84 {
	my $self = shift;
	my $acc  = shift;
    $acc = $self->ExpandAccount($acc);
	my $h;
    if (substr($acc, 2, 1) != '9') {
		$h = $self->Mark33($acc);
		if ($h == 1) {
			$h = $self->Method06($acc, '000065432', 0, 10, 7);
		}
    } else {
        $h = $self->Mark51($acc);
    }
	my $mark84 = $h;
    return $mark84;
}

sub Mark85 {
	my $self = shift;
	my $acc  = shift;
  	$acc = $self->ExpandAccount($acc);
    my $h = $self->Method06($acc, '000065432', 0, 10, 11);
    if ($h == 1) {
		if (substr($acc, 2, 2) == '99') {
			$h = $self->Method02($acc, '008765432', 0);
		} else {
			$h = $self->Method06($acc, '000065432', 0, 10, 7);
		}
    }
    my $mark85 = $h;
	return $mark85;
}

sub Mark86 {
	my $self = shift;
	my $acc  = shift;
    my $h = $self->Method00($acc, '000121212', 10);
    if ($h == 1) {
		if (substr($acc,2,1) == '9') {
			$h = $self->Method06($acc, 'A98765432', 1, 10, 11);
		} else {
			$h = $self->Method06($acc, '000765432', 0, 10, 11);
		}
    }
    my $mark86 = $h;
    return $mark86;
}


## This method is incorrect in original. Approved by default
##
## sub Mark87 {
## 	my $self = shift;
## 	my $acc  = shift;
## 	my @Tab1 = (0,4,3,2,6);
## 	my @Tab2 = (7,1,5,9,8);
## 
##     my $Result = 1;
##     $acc = $self->ExpandAccount($acc);
##     if (substr($acc,2,1) == '9') {
## 		$Result = $self->Mark10($acc);
##     } else {
## 		my @accTemp;
## 		$accTemp[$_] = int(substr($acc,$_,1)) for (0..(length($acc))-1);
## 		my $i = 4;
## 		while ($accTemp[$i] == 0) {
## 			$i++;
## 		}
## 		my $C2 = $i % 2;
## 		my $D2 = 0;
## 		my $A5 = 0;
## 		while ($i < 10) {
## 			if ($accTemp[$i] == 0) {
## 				$accTemp[$i] = 5;
## 			} elsif ($accTemp[$i] == 1) {
## 				$accTemp[$i] = 6;
## 			} elsif ($accTemp[$i] == 5) {
## 				$accTemp[$i] = 10;
## 			} elsif ($accTemp[$i] == 6) {
## 				$accTemp[$i] = 1;
## 			}
## 
## 			if ($C2 == $D2) {
## 				if ($accTemp[$i] > 5) {
## 					if (($C2 == 0) && ($D2 == 0)) {
## 						$C2 = 1;
## 						$D2 = 1;
## 						$A5 = $A5 + 6 - ($accTemp[$i] - 6);
## 					} else {
## 						$C2 = 0;
## 						$D2 = 0;
## 						$A5 = $A5 + $accTemp[$i];
## 					}
## 				} else {
## 					if (($C2 == 0) && ($D2 == 0)) {
## 						$C2 = 1;
## 						$A5 = $A5 + $accTemp[$i];
## 					} else {
## 						$C2 = 0;
## 						$A5 = $A5 + $accTemp[$i];
## 					}
## 				}
## 			} else {
## 				if ($accTemp[$i] > 5) {
## 					if ($C2 == 0) {
## 						$C2 = 1;
## 						$D2 = 0;
## 						$A5 = $A5 - 6 + ($accTemp[$i] - 6);
## 					} else {
## 						$C2 = 0;
## 						$D2 = 1;
## 						$A5 = $A5 - $accTemp[$i];
## 					}
## 				} else {
## 					if ($C2 == 0) {
## 						$C2 = 1;
## 						$A5 = $A5 - $accTemp[$i];
## 					} else {
## 						$C2 = 0;
## 						$A5 = $A5 - $accTemp[$i];
## 					}
## 				}
## 			}
## 			$i++;
## 		}
## 		while (($A5 < 0) || ($A5 > 4)) {
## 			if ($A5 > 4) {
## 				$A5 = $A5 - 5;
## 			} else {
## 				$A5 = $A5 + 5;
## 			}
## 		}
## 		my $P;
## 		if ($D2 == 0) {
## 			$P = $Tab1[$A5];
## 		} else {
## 			$P = $Tab2[$A5];
## 		}
## 		if ($P == $accTemp[9]) {
## 			$Result = 0;
## 		} else {
## 			if ($accTemp[4] == 0) {
## 				if ($P > 4) {
## 					$P = $P - 5;
## 				} else {
## 					$P = $P + 5;
## 				}
## 				if ($P == $accTemp[9] ) {
## 					$Result = 0;
## 				}
## 			}
## 		}
## 		if ($Result != 0 ) {
## 			$Result = $self->Mark33($acc);
## 			if ($Result != 0 ) {
## 				$Result = $self->Method06($acc,'000065432',0,10,7);
## 			}
## 		}
##     }
##     return $Result;
## }

sub Mark88 {
	my $self = shift;
	my $acc  = shift;
    $acc = $self->ExpandAccount($acc);
	my $mark88;
    if (substr($acc,2,1) eq '9') {
		$mark88 = $self->Method06($acc, '008765432', 0, 10, 11);
    } else {
		$mark88 = $self->Method06($acc, '000765432', 0, 10, 11);
    }
    return $mark88;
}

sub Mark89 {
	my $self = shift;
	my $acc  = shift;
    my $Correct = 1;
	my $length = length($acc);
	if (map {$_ == $length} (1,2,3,4,5,6,10)) {
		$Correct = 4;
	} elsif (map {$_ == $length} (7,9)) {
		$acc = $self->ExpandAccount($acc);
		$Correct = $self->Method06($acc,'098765432',0,10,11);
	} else {
		if (((int($acc) > 32000005) && (int($acc) < 38999995)) || ((int($acc) >1999999999) && (int($acc) < 400000000))) {
			$Correct = $self->Mark10($acc);
		}
	}
    return $Correct;
}

sub Mark90 {
	my $self = shift;
	my $acc  = shift;
    my $h = $self->Method06($acc, '000765432', 0, 10, 11);
	if ($h != 0) {
		$h = $self->Method06($acc, '000065432', 0, 10, 11);
		if ($h != 0) {
			my $str = substr($acc,-1);
			if (map {$_ == $str} (7,8,9)) {
				$h = 4;
			} else {
				$h = $self->Method06($acc, '000065432', 0, 10, 7);
			}
		}
		$h = $self->Method06($acc, '000065432',0, 10, 9) if ($h != 0);
		$h = $self->Method06($acc, '000021212',0, 10, 10) if ($h != 0);
	}
    return $h;
}

sub Mark91 {
	my $self = shift;
	my $acc  = shift;
    my $h = $self->Method06($acc, '765432', 0, 7, 11);
    if ($h == 1) {
		$h = $self->Method06($acc, '234567', 0, 7, 11);
		if ($h == 1) {
			$h = $self->Method06($acc, 'A987650432', 1, 7, 11);
			if ($h == 1) {
				$h = $self->Method06($acc, '9A5842', 1, 7, 11);
			}
		}
    }
    return $h;;
}

sub Mark92 {
	my $self = shift;
	my $acc  = shift;
    my $mark92 = $self->Method01($acc, '000173173');
    return $mark92;
}

sub Mark93 {
	my $self = shift;
	my $acc  = shift;
    $acc = $self->ExpandAccount($acc);
	my $correct = 1;
	if (substr($acc,0,4) == '0000') {
        $correct = $self->Method06($acc,'000065432',0,10,11);
	} else {
        $correct = $self->Method06($acc,'65432',0,10,11);
    }
	if ($correct == 1) {
        if (substr($acc,0,4) == '0000') {
			$correct = $self->Method06($acc,'000065432',0,10,7);
        } else {
			$correct = $self->Method06($acc,'65432',0,10,7);
		}
    }
    my $mark93 = $correct;
    return $mark93;
}

sub Mark94 {
	my $self = shift;
	my $acc  = shift;
    my $mark94 = $self->Method00($acc, '121212121', 10);
    return $mark94;
}

sub Mark95 {
	my $self = shift;
	my $acc  = shift;
	my $mark95;
    if (length($acc) == 10) {
		if ((substr($acc,3,1) == '5') || (substr($acc, 3, 2) == '69')) {
			$mark95 = $self->Method06($acc, '000765432', 0, 10, 11);
		} else {
			$mark95 = $self->Method06($acc, '432765432', 0, 10, 11);
		}
    } else {
		$mark95 = $self->Method06($acc, '432765432', 0, 10, 11);
		if (int($acc) < 2000000) {
			$mark95 = 4;
		}
		if (int($acc) > 699999999) {
			if (int($acc) < 800000000) {
				$mark95 = 4;
			}
		}
		if (int($acc) > 395999999) {
			if (int($acc) < 500000000) {
				$mark95 = 4;
			}
		}
    }
    return $mark95;
}

sub Mark96 {
	my $self = shift;
	my $acc  = shift;
    $acc = $self->ExpandAccount($acc);
    my $h = $self->Mark19($acc);
	if ($h == 1) {
        $h = $self->Method00($acc, '212121212', 10);
        if ($h == 1) {
			if (int($acc) >1299999) {
				if (int($acc) < 99400000) {
					$h = 0;
				}
			}
        }
	}
	my $mark96 = $h;
	return $mark96;
}

sub Mark97 {
	my $self = shift;
	my $acc  = shift;
    $acc = $self->ExpandAccount($acc);
    my $h = int(substr($acc, 0, 9)) % 11;
	my $mark97;
	if ($h == 10) {
        $h = 0;
	}
	if (substr($acc,-1) == $h) {
        $mark97 = 0;
	} else {
        $mark97 = 1;
	}
	return $mark97;
}

sub Mark98 {
	my $self = shift;
	my $acc  = shift;
    $acc = $self->ExpandAccount($acc);
    my $mark98 = $self->Method01($acc,'003713713');
	$mark98 = $self->Mark32($acc) if ($mark98 == 1);
    return $mark98;
}

sub Mark99 {
	my $self = shift;
	my $acc  = shift;
    my $mark99 = $self->Method06($acc, '432765432', 0, 10, 11);
    if (int($acc) >= 396000000 && int($acc) <= 499999999) {
		$mark99 = 4;
	}
    return $mark99;
}

sub MarkA1 {
	my $self = shift;
	my $acc  = shift;
	my $RetVal;
    if (length($acc) == 8 || length($acc)==10) {
		$acc = $self->ExpandAccount($acc);
		$RetVal = $self->Method00($acc, '002121212', 10);
    } else {
		$RetVal = 1;
    }
    return $RetVal;
}

sub MarkA2 {
	my $self = shift;
	my $acc  = shift;
    $acc = $self->ExpandAccount($acc);
    my $RetVal = $self->Method00($acc, '212121212', 10);
    if ($RetVal != 0) {
		$RetVal = $self->Mark04($acc);
    }
    return $RetVal;
}

sub MarkA3 {
	my $self = shift;
	my $acc  = shift;
    $acc = $self->ExpandAccount($acc);
    my $RetVal = $self->Method00($acc, '212121212', 10);
    if ($RetVal != 0) {
		$RetVal = $self->Mark10($acc);
    }
    return $RetVal;
}

sub MarkA4 {
	my $self = shift;
	my $acc  = shift;
    $acc = $self->ExpandAccount($acc);
	my $RetVal;
    if (int(substr($acc, 2,2)) != 99) {
		$RetVal = $self->Method06($acc,'000765432',0, 10,11);
		if ($RetVal !=0) {
			my $sig='000765432';
			$RetVal = 1;
			my $h = 0;
			$h += substr($acc,$_,1) * substr($sig,$_,1) for (0..(length($sig)-1));
			$h = $h % 7;
			my $checksum = 7 - $h;
			$checksum = 0 if ($h == 0);
			$RetVal = 0 if ($checksum == substr($acc,-1));
		}
		$RetVal = $self->Mark93($acc) if ($RetVal != 0);
    } else {
		$RetVal = $self->Method06($acc,'000065432',0,10,11);
		$RetVal = $self->Mark93($acc) if ($RetVal != 0);
    }
    return $RetVal;
}

sub MarkA5 {
	my $self = shift;
	my $acc  = shift;
    $acc = $self->ExpandAccount($acc);
    my $RetVal = $self->Method00($acc, '212121212', 10);
    if ($RetVal != 0) {
		if (substr($acc,1,1) != "9") {
			$RetVal = $self->Mark10($acc);
		} else {
			$RetVal = 1;
		}
    }
    return $RetVal;
}

sub MarkA6 {
	my $self = shift;
	my $acc  = shift;
    $acc = $self->ExpandAccount($acc);
	my $RetVal;
    if (substr($acc,1,1) != "8") {
		$RetVal = $self->Method01($acc, '173173173');
    } else {
		$RetVal = $self->Method00($acc, '212121212', 10);
    }
    return $RetVal;
}

sub MarkA7 {
	my $self = shift;
	my $acc  = shift;
    $acc = $self->ExpandAccount($acc);
    my $RetVal = $self->Method00($acc, '212121212', 10);
    if ($RetVal != 0) {
		$RetVal = $self->Mark03($acc);
    }
    return $RetVal;
}

sub MarkA8 {
	my $self = shift;
	my $acc  = shift;
    $acc = $self->ExpandAccount($acc);
    my $RetVal = $self->Mark81($acc);
    if ($RetVal != 0) {
		if (substr($acc,2,1) != "9") {
			$RetVal = $self->Mark73($acc);
		} else {
			$RetVal = 1;
		}
    }
    return $RetVal;
}

sub MarkA9 {
	my $self = shift;
	my $acc  = shift;
    $acc = $self->ExpandAccount($acc);
    my $RetVal = $self->Method01($acc, '173173173');
    if ($RetVal != 0) {
		$RetVal = $self->Method06($acc, '432765432', 0, 10, 11);
		if ($RetVal != 0) {
		}
    }
    return $RetVal;
}

##################################
##### Marks end ##################
##################################

1;
