package HSPC::MT::Plugin::PP::BT_DTAUS;

use strict;
use HSPC::PluginToolkit::General qw(split_date_string datetime_gmt_now gmt_to_local string throw_exception log log_debug log_warn round2);
use Encode qw(from_to);
use HSPC::MT::Plugin::PP::BT_DTAUS::Tools;


use constant DTAUS_BINARY			=> '/usr/share/plugin-pp-bt-dtaus/dtaus';
use constant IS_WINDOWS	=> $^O =~ /^(?:mswin|dos|os2)/oi;


sub get_title {
	my $class = shift;
	return	string('dtaus_naming');
}

sub process_batch_content {
	my $self = shift;
	my %h = (
		config           => undef,
		transaction_list => undef,
		vendor_info      => undef,
		file_id          => undef,
		@_
	);

	my $config				= $h{config};
	my $transaction_list	= $h{transaction_list};
	my $file_id				= $h{file_id};

	if (
		!length( $h{config}->{merch_name} )
		||
		!length( $h{config}->{bank_code} )
		||
		!length( $h{config}->{acc_number} )
	) {
		throw_exception (
			type		=> 'error',
			log_message	=> string('dtaus_not_configured_error'),
		);
	}	

	my $cdate		= gmt_to_local( datetime_gmt_now() );
	my @tmp			= split( /\s/, $cdate );
	my @tmp2		= split( /-/, $tmp[0] );
	my $today_date	= $tmp2[2].$tmp2[1].substr($tmp2[0],2,2);

	my $batch_content = "";
	
	## First of all get "A" (header) part
	$batch_content	= HSPC::MT::Plugin::PP::BT_DTAUS::Tools->construct_a_part ( 
		provider_bank_code	=> $config->{bank_code},
		provider_acc_number	=> $config->{acc_number},
		provider_merch_name	=> $config->{merch_name},
		today_date			=> $today_date,
	);

	my $bank_code_summ  = 0;
	my $acc_number_summ = 0;
	my $result_amount   = 0;
	my $number_of_c_records = 0;

	foreach my $t (@$transaction_list) {
		my $pd					= $t->{payment_method}->{public_data};
		my $sd					= $t->{payment_method}->{secure_data};

		my $amount_to_be_paid	= $t->{transaction_amount};
		$amount_to_be_paid		= round2( $amount_to_be_paid );

		log "Amount to be PAID = " . $amount_to_be_paid;

		$batch_content	.= HSPC::MT::Plugin::PP::BT_DTAUS::Tools->one_c_record (
			provider_bank_code  => $config->{bank_code},
			provider_acc_number => $config->{acc_number},
			provider_merch_name => $config->{merch_name},
			customer_bank_code  => $sd->{bank_code},
			customer_acc_number => $sd->{account_number},
			customer_merch_name => $pd->{holder_name},
			customer_country	=> $t->{account_info}->{address_country},
			amount              => $amount_to_be_paid,
			payment_purpose     => string('order_desc_simple', order_id => $t->{document_info}->{id}),
		);

		$bank_code_summ			+= $sd->{bank_code};
		$acc_number_summ		+= $sd->{account_number};
		$result_amount			+= $amount_to_be_paid;
		$number_of_c_records++;
	}

	## and finally close batch file with "E"(footer) record
	$batch_content	.= HSPC::MT::Plugin::PP::BT_DTAUS::Tools->construct_e_part ( 
			provider_bank_code  => $config->{bank_code},
			provider_acc_number => $config->{acc_number},
			bank_code_summ      => $bank_code_summ,
			acc_number_summ     => $acc_number_summ,
			result_amount       => round2($result_amount),
			c_number            => $number_of_c_records,
	);
	from_to($batch_content,'utf8','latin1');

	my $binary	= &DTAUS_BINARY();
	my $cmd;
	my $tmp;
	if ( IS_WINDOWS ) {
		require File::Basename;
		require Cwd;
		my $binary = Cwd::realpath(File::Basename::dirname(__FILE__)) . '\BT_DTAUS\dtaus.exe';

		require File::Temp;
		$tmp = File::Temp->new();
		my $tempname = $tmp->filename;
       	
		print $tmp $batch_content;

		$cmd = "cmd /c \"$binary\" -dtaus -d - -c - -b NUL -o NUL 2>&1 < $tempname";
	} else {
		$cmd = 'echo -n ' . $self->__shell_safe_string(string=>$batch_content) .
			' | ' . &DTAUS_BINARY() . ' -dtaus -d - -c - -b /dev/null -o /dev/null 2>&1';
		
	}

	log_debug "command for dtaus:\n$cmd";

	my $res;
	eval {
		$res	= `$cmd`;
	};
	if( $@ ) {
		log_warn "System error during batch create, : $@";
		throw_exception (
			type		=> 'error',
			log_message	=> string('dtaus_cant_create_batch_system_error', error=>$@),
		);
	} elsif( length($res) < 100 ) {
		log_warn "Error during batch create, dtaus binary returns: $res";
		throw_exception (
			type		=> 'error',
			log_message	=> string('dtaus_cant_create_batch_dtaus_error', result=>$res),
		);
	}

	return {
		BATCH_CONTENT   => $res,
		BATCH_FILE_NAME => sprintf("dtaus_file_%06d.txt", $file_id),
	};
};

sub get_supported_payment_method_types{
	my $class = shift;
	return [
		'BT_DTAUS',
	];
}

sub get_currencies_supported{
	return [
		'EUR', ## Euro
	];
}

## private method
## produces shell safe string parameter
## >> self
## =>> string
## <	safe string
sub __shell_safe_string {
	my ($self,%h)	= (@_);
	$h{string}		=~ s#'#'\\''#g;
	return	"'$h{string}'";
}

1;
