package HSPC::MT::Plugin::PP::BT_DTAUS::Tools;

use strict;


sub new {
    my $class = shift;
    my $self = {};
    bless ($self,$class);

    return $self;
}

## constructs "A" (header) part of DTAUS file
## parameters:
##  =>> provider_bank_code  
##  =>> provider_acc_number 
##  =>> provider_merch_name  
##  =>> today_date
##  =>> customer_ref_number
## return value:
##  << ready "A" record
sub construct_a_part {
	my $class = shift;
	my %h = @_;

	my $provider_bank_code  = $h{provider_bank_code};
	my $provider_acc_number = $h{provider_acc_number};
	my $provider_merch_name = $h{provider_merch_name};
	my $today_date          = $h{today_date};
	my $customer_ref_number = $h{customer_ref_number};

	## cut of provider name if needed
	$provider_merch_name	= substr( $provider_merch_name, 0, 27 )
		if length($provider_merch_name) > 27;

	my $a = "BEGIN { \n";
	$a .= "  Art	LK\n";
	$a .= "  Name	$provider_merch_name\n";
	$a .= "  Konto	$provider_acc_number\n";
	$a .= "  BLZ	$provider_bank_code\n";
	$a .= "  Datum	$provider_bank_code\n";
	$a .= "  Datum	$today_date\n";
  	$a .= "  Euro\n}\n";
	
	return $a;
}

## constructs "C" (or header) part of DTAUS file
## parameters:
##  =>> provider_bank_code  
##  =>> provider_acc_number 
##  =>> provider_merch_name  
##  =>> customer_bank_code  
##  =>> customer_acc_number 
##  =>> customer_merch_number
##  =>> customer_country
##  =>> amount       
##  =>> payment_purpose
## return value:
##  << ready "C" record
sub one_c_record {
	my $class = shift;
	my %h = @_;
	my $provider_bank_code  = $h{provider_bank_code};
	my $provider_merch_name = $h{provider_merch_name};
	my $provider_acc_number = $h{provider_acc_number};

	my $customer_bank_code  = $h{customer_bank_code};
	my $customer_acc_number = $h{customer_acc_number};
	my $customer_merch_name = $h{customer_merch_name};
	my $customer_country   = $h{customer_country};
	my $amount              = $h{amount};
	my $payment_purpose     = $h{payment_purpose};

	## cut of provider name if needed
	$provider_merch_name	= substr( $provider_merch_name, 0, 27 )
		if length($provider_merch_name) > 27;

	## cut of customer name if needed
	$customer_merch_name	= substr( $customer_merch_name, 0, 27 )
		if length($customer_merch_name) > 27;

	my $c = "{\n";
	$c .= "  Name	$customer_merch_name\n";
	$c .= "  Konto	$customer_acc_number\n";
	$c .= "  BLZ	$customer_bank_code\n";
	$c .= "  Transaktion	Einzug\n";
	$c .= "  Betrag	$amount\n";
	$c .= "  Zweck	$customer_merch_name\n";
	$c .= "  myName	$provider_merch_name\n";
	$c .= "  myKonto	$provider_acc_number\n";
	$c .= "  myBLZ	$provider_bank_code\n";
	$c .= "  Text	$payment_purpose\n";
	$c .= "}\n";

	return $c;
}

## Constructs "E" part of DTAUS file
## Parameters:
##  =>> c_number  -- number of "C" records
##  =>> provider_bank_code  
##  =>> provider_acc_number 
##  =>> bank_code_summ  
##  =>> acc_number_summ
##  =>> result_amount       
## Return value:
##  << ready "E" record
sub construct_e_part {
	my $class = shift;
	my %h = @_;
	my $c_number = $h{c_number};
	my $acc_number_summ = $h{acc_number_summ};
	my $bank_code_summ = $h{bank_code_summ};
	my $result_amount = $h{result_amount};

	my $e = "END {\n";
	$e .= "  Anzahl	$c_number\n";
	$e .= "  Summe	$result_amount\n";
	$e .= "  Kontos	$acc_number_summ\n";
	$e .= "  BLZs	$bank_code_summ\n";
	$e .= "}";

	return $e;
}

1;
