package HSPC::Plugin::PP::BT_DTAUS;

use strict;
use HSPC::PluginToolkit::HTMLTemplate qw(parse_template);
use HSPC::PluginToolkit::General qw(string argparam throw_exception get_help_url);

sub view_form {
	my $class = shift;
	my %h    = (
		config => undef,
		@_
	);
	my $config = $h{config};
	my $html;

	my %tmpl_args = (
		merch_name	=> $config->{merch_name},
		bank_code	=> $config->{bank_code},
		acc_number	=> $config->{acc_number},
	);
	$html = parse_template(
				path => __PACKAGE__,
				name => 'bt_dtaus_view.tmpl',
				data => \%tmpl_args,
		);

	return $html;
}

sub edit_form {
	my $class = shift;
	my %h    = (
		config  => undef,
		@_
	);
	my $html;
	my $config = $h{config};
	
	my %tmpl_args = (
		merch_name	=> argparam('merch_name')	|| $config->{merch_name},
		bank_code	=> argparam('bank_code')	|| $config->{bank_code},
		acc_number	=> argparam('acc_number')	|| $config->{acc_number},
	);
	$html = parse_template(
				path => __PACKAGE__,
				name => 'bt_dtaus_edit.tmpl',
				data => \%tmpl_args,
		);

	return $html;
}

sub collect_data {
	my $class = shift;
	my %h    = (
		config  => undef,
		@_
	);
	my $config = $h{config};
	
	## mandatory params are marked with value > 0
	my $params	= {
		merch_name		=> 1,
		bank_code		=> 1,
		acc_number		=> 1,
	};
	my $error;
	foreach my $param ( keys(%$params) ) {
		my $value	= argparam($param);
		if( $value ) {
			$config->{$param}	= $value;
		} elsif( $params->{$param} ) {
			$error	.= string( 'dtaus_' . $param . '_must_be_defined' ) . "<br>\n";
		}
	}
	throw_exception (
		type			=> 'error',
		vendor_message	=> $error,
	)	if $error;


	## validate bank_code & acc_number
	my @errors;

	my $merch_name	= $config->{merch_name};
	if( length($merch_name) > 27 ) {
		## do nothing here, but while batch generation such name will be cutted to 27 symbols
	}

	my $acc_number  = $config->{acc_number};
	unless( length($acc_number) ) {
		push( @errors, string('dtaus_pm_account_number_must_be_defined') );
	}
	if( $acc_number =~ /\D/ ) {
		push( @errors, string('dtaus_pm_account_number_non_digit', acc=>$acc_number) );
	}
	if( length($acc_number) > 10 ) {
		push( @errors, string('dtaus_pm_account_number_length10') );
	}

	my $bank_code   = $config->{bank_code};
	unless( length($bank_code) ) {
		push( @errors, string('dtaus_pm_bank_code_must_be_defined') );
	}
	if( length($bank_code) > 8 ) {
		push( @errors, string('dtaus_pm_bank_code_length8') );
	}

	unless( scalar(@errors) ) {
		my $validator   = HSPC::MT::Plugin::PM::BT_DTAUS::Valid->new();
		push( @errors, string('dtaus_pm_bank_code_wrong') )
			unless $validator->IsAccountValid($acc_number,$bank_code);
	}

	throw_exception(
		type    => 'parameter',
		message => join( "<br>\n", @errors ),)
		if scalar(@errors);
	
	return $config;
}

sub get_help_page {
	my $class = shift;
	my %h    = (
		action  => undef,
		language  => undef,
		@_
	);

	my $action   = $h{action};
	my $language = $h{language};

	my  $html = parse_template(
				path => __PACKAGE__ . '::' . uc($language),
				name => "pp_help_$action.html",
				data => {
					about_url =>
					  get_help_url( action => 'about', language => $language, )
				},
			);

	return $html;
}

1;
