#!/usr/bin/perl

use strict;
use HSPC::WebDB;
use HSPC::MT::CCP::Constants qw(:trans_types);
use HSPC::MT::PluginConfig::PP;

use constant PLUGIN_ID   => 'BT_DTAUS';
use constant TEMPLATE_ID => 'BT_DTAUS';

if (is_table_exists(table=>'vendor_pp_plugin') 
	&& is_table_exists(table=>'pp_plugin_dtaus') )
{
	my $rows = 
	  select_hashrows(q|
			SELECT
				vendor_pp_plugin.vendor_id as vendor_id,
				vendor_pp_plugin.plugin_id as plugin_id,
				vendor_pp_plugin.is_active as is_active,
				vendor_pp_plugin.banner_text banner_text,
				pp_plugin.title as title,
				pp_plugin.is_autopay_supported as is_autopay_supported,

				pp_plugin_dtaus.bank_code   bank_code,
				pp_plugin_dtaus.acc_number  acc_number,
				pp_plugin_dtaus.merch_name  merch_name

			FROM 
				vendor_pp_plugin, pp_plugin, pp_plugin_dtaus
			WHERE
					vendor_pp_plugin.plugin_id = pp_plugin.id
					and 
					vendor_pp_plugin.plugin_id = pp_plugin_dtaus.plugin_id
					and 
					vendor_pp_plugin.vendor_id = pp_plugin_dtaus.vendor_id
					and 
					vendor_pp_plugin.plugin_id = ?
		|, &PLUGIN_ID );

	exit 0 unless $rows;

	foreach my $plugin (@$rows) {

		## is configured ?
		if ( $plugin->{acc_number} && $plugin->{bank_code} && $plugin->{merch_name} ) {
			my $new_plugin = HSPC::MT::PluginConfig::PP->new();

			$new_plugin->vendor_id( $plugin->{vendor_id} );
			$new_plugin->is_active( $plugin->{is_active} );
			$new_plugin->plugin_type('PP');
			$new_plugin->name( $plugin->{title} );

			my $config = {
					acc_number 	=> $plugin->{acc_number},
					bank_code 	=> $plugin->{bank_code},
					merch_name 	=> $plugin->{merch_name},
			};
			$new_plugin->config_data( $config );

			$new_plugin->template_id( &TEMPLATE_ID );

			$new_plugin->banner_text( $plugin->{banner_text} );

			$new_plugin->is_direct( $plugin->{is_autopay_supported} );
			$new_plugin->save();

			if ($new_plugin->id()) {
				select_run(q|
					insert into plugin_pp_paytypes (config_id, type, template_id) values (?,?,?)|,
					$new_plugin->id(), 'BT_DTAUS', 'BT_DTAUS'
				);
			
				## upgrade orders payed by this plugin
				select_run(q|
					UPDATE
						ar_doc_rb, ar_doc
					SET
						ar_doc_rb.plugin_id = ? 
					WHERE
						ar_doc_rb.ar_doc_id = ar_doc.ar_doc_id 
						AND ar_doc.vendor = ?
						AND ar_doc_rb.plugin_id = ?|, $new_plugin->id(), $new_plugin->vendor_id(), &PLUGIN_ID );

				## upgrade batches generated by this plugin
				select_run(q|
					UPDATE
						bm_doc_batch
					SET
						plugin_id = ?
					WHERE
						vendor_id = ?
						AND plugin_id = ?|,
					$new_plugin->id(), $new_plugin->vendor_id(), &PLUGIN_ID
				);

				## Update TransLog
				select_run(q|
					UPDATE
						pp_trans_log, account
					SET
						pp_trans_log.plugin_id = ?
					WHERE
						pp_trans_log.account_no = account.account_no
						AND account.vendor = ?
						AND pp_trans_log.plugin_id = ?|, $new_plugin->id(), $new_plugin->vendor_id(), &PLUGIN_ID 
				);
			}
		}
	}
}

## update reseller config
select_run(
	q|UPDATE reseller_config_plugin SET plugin_id = ? WHERE plugin_id = ?|,
	&TEMPLATE_ID,
	&PLUGIN_ID
);

